;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x3566ce290cb3788a393635ad531a29fe . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x8c8a74e6401f0001f5dedf19833aa862 . "./code.cg2js/cg-printing.fasl")
(#x60637ed8cef97303c3ead804ddfe0443 . "./code.cg2js/cg-caret.fasl")
(#x506d68d26ce284d222844f1ceeefcc2c . "./code.cg2js/cg-item-list.fasl")
(#xb9d0a70966bae9c9d00962c09dcd92a5 . "./code.cg2js/cg-progress-indicator.fasl")
(#xe1193273011786c06173ca29ecea8839 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xd11db0c79457924eccde192234698297 . "./code.cg2js/cg-outline.fasl")
(#xd4bc96acfff7298e46a33b191ed48eda . "./code.cg2js/cg-palette.fasl")
(#x0285334d4d2772a341a58e255cd6f6e6 . "./code.cg2js/cg-selected-object.fasl")
(#xd55c490f9e33d95cba28554797f5d3b6 . "./code.cg2js/cg-status-bar.fasl")
(#xe5c5338b8657fdecd488bc813be90009 . "./code.cg2js/cg-wrap-string.fasl")
(#xb555aca2d4d9c226fbbbba35bd40c34e . "./code.cg2js/cg-fill-texture.fasl")
(#x57f6f7c9389c6121d5a49d0e3774cd36 . "./code.cg2js/cg-layout.fasl")
(#x0aff26ea8d9006f7e0796e102c8f8308 . "./code.cg2js/cg-timer.fasl")
(#x1e20433056697cb20a7e8d0b92974fca . "./code.cg2js/cg-dialog-item.fasl")
(#x414329146733ef95bcdf5bd072f1de5a . "./code.cg2js/cg-button.fasl")
(#x14e4fd0163e0c1608accd766f1a2356e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xe839af03ead2f1b9217f6bab175504db . "./code.cg2js/cg-tray.fasl")
(#xa00ff2724f447634bd223e38fe4a6db4 . "./code.cg2js/cg-plot-widget.fasl")
(#xf72439908d13f7df4ac05500af2a7150 . "./code.cg2js/cg-pixmap.fasl")
(#x21eff2d0ff1ed6f7b84f852d9caa3bcb . "./code.cg2js/cg-font-dialog.fasl")
(#xf52e10e3f05cab2c18b948bb522924f2 . "./code.cg2js/cg-os-widget.fasl")
(#xa89cb71f91a036463ddc0388b4a5ee5a . "./code.cg2js/cg-text-or-combo.fasl")
(#x4282f7c1610c20948e5612cf54d81154 . "./code.cg2js/cg-trackbar.fasl")
(#x1591560eba5c1df5e1c7626880de75e1 . "./code.cg2js/cg-up-down-control.fasl")
(#x951b5c7d8afe3761cfe7ba2df7109a48 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x119944caf9f72f451bf2ad450d282fab . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x0e3c227fe99218bfbf41589004fd0d54 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x262961cd74ef8e17e835e692700bf21e . "./code.cg2js/cg-bitmap-stream.fasl")
(#x91a4b034860d4e3c9c9ced4bd75b1f6a . "./code.cg2js/cg-chart-widget.fasl")
(#xac5d986813436d3893c6197ab4197f0f . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x3bfed8c05350911be28e4e04e25a6bc8 . "./code.cg2js/cg-group-box.fasl")
(#xefc097294586e6cba0de5d9223356dfd . "./code.cg2js/cg-static-text.fasl")
(#xe8e07cc9ab67430b96acce56437a56ad . "./code.cg2js/cg-class-slot-grid.fasl")
(#x3ab98a90ea3592b0c82637d9a783173a . "./code.cg2js/cg-color-dialog.fasl")
(#xaab682ddff3a53c70e2bed49ae76f999 . "./code.cg2js/cg-intersect.fasl")
(#x9526f4d3e6eb470690ccf3afe2e086ef . "./code.cg2js/cg-gesture-emulation.fasl")
(#x7bf4d908581784bdab6870ea41cbfd8c . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xac0caa206c6bca2e99d222694da912eb . "./code.cg2js/cg-html-dialog.fasl")
(#x5aab4ce7989bd2c29a07c12fc7c26585 . "./code.cg2js/cg-lettered-menu.fasl")
(#xa37d60b1cb143104171ac4ee04799fe5 . "./code.cg2js/cg-class-grid.fasl")
(#x58315ea4c50c2f6a8c48535bbc707c84 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd4dfbc51916e5eeae55ad79f95d63878 . "./code.cg2js/cg-calendar.fasl")
(#x8fbd6a6dee7a55f1c6f1dbe6026f3334 . "./code.cg2js/cg-editable-text.fasl")
(#x05461be8234ad59c2bf8ed5cf9b1de3f . "./code.cg2js/cg-property.fasl")
(#xa86610dfb463aaeca447f8618a929c9c . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x92e0526df5184836b52c105a7291e143 . "./code.cg2js/cg-grid-widget.fasl")
(#x971de7036b293beb11f31524ad4c08fa . "./code.cg2js/cg-paren-matching.fasl")
(#xecf172c2a459e817d33b05021cea0831 . "./code.cg2js/cg-dev.fasl")
(#x48feae631f914fd86a0e871b0034c3ea . "./code.cg2js/cg-drawable.fasl")
(#xaf820eae5ebf5258351aac986e750b79 . "./code.cg2js/cg-curve.fasl")
(#x475215bfa5681b14fa1eb2889f7388a2 . "./code.cg2js/cg-lisp-widget.fasl")
(#x915aeb7a0c68d99fd2c4085769d48282 . "./code.cg2js/cg-dde-utils.fasl")
(#x1a753c6a9f0f2eace5972f33c25b2bff . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x0f606ba244835ee7e2297c1e71b96886 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x196102e45118f459d6c34560d5a61c25 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x9d15f628aaf62b8a5d8b1476ce2ec438 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x7be8a39f01c3536998f945b7af72a3ad . "./code.cg2js/cg-lamp.fasl")
(#x74e01eecf419ae0f357adf3c74bc50a3 . "./code.cg2js/cg-common-control.fasl")
(#xb442c8abd6d8f593d92ba35135265c46 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc1277c43eb839fb31caa612e0fb7508f . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x55989c0ecd0126acbc27c3e91919f546 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xd08da5852111a8af169769c6a090d603 . "./code.cg2js/cg-project-window.fasl")
(#x8ef4d63617c4fb36708812876540a830 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x2d9648da1ded34a06a23591472a18b8e . "./code.cg2js/cg-class-support.fasl")
(#xcdadc8ab625a3dec48a5f0622071d104 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x9b77c2595f126ba93e9ac08f3c985f86 . "./code.cg2js/cg-yes-no-list.fasl")
(#x33e0d8353b05f97499ca21c79e7640c3 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x813a6d4b81e12fa883d6826ee434fb80 . "./code.cg2js/cg-string-dialog.fasl")
(#x5b341bbf03c90492c07e0c14ad7d10fc . "./code.cg2js/cg-menu-tooltip.fasl")
(#x4e35837bd23669884efac4cea3793286 . "./code.cg2js/cg-dropping-outline.fasl")
(#xb08ba09ccf4fca0a332336ff8d645981 . "./code.cg2js/cg.fasl")
(#x69136ce832254d5e206cfb30ec440fd5 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x4f2ac7743710b94be4f6ba0ae1d9b2a8 . "./code.cg2js/cg-scaling-stream.fasl")
(#xbad3a4b3d09fc1d226785aa403d12691 . "./code.cg2js/cg-yes-no-string.fasl")
(#xb04cba2fefc88c9599ad110cf66d2718 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xfc6962bf09b3d6538c4623d4373ab58c . "./code.cg2js/cg-web-browser.fasl")
(#xdf60de4130ead569078b71ea4f3ff7fd . "./code.cg2js/cg-clipboard-stack.fasl")
(#xfc372422f206a9c17b145fb20052463e . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb73530512e9ff0e3a68067047627a4d0 . "./code.cg2js/cg-message-dialog.fasl")
(#xd6c47d4eb325c1289eaa123c1587682b . "./code.cg2js/cg-comtab.fasl")
(#x6915e9606c49b1b5b5877e7eee062464 . "./code.cg2js/cg-base.fasl")
(#x8b7bc378d1e6b38c20eb10e928229cd0 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xd9ef2f7f18b5c3ecbb1595522aba045d . "./code.cg2js/cg-text-edit-pane.fasl")
(#x3b79052cedee22cef74b24db53c2fd6e . "./code.cg2js/cg-utility-dialog.fasl")
(#xedd5fa1b554fa1a8ad6f7610dad55c7f . "./code.cg2js/cg-object-editor.fasl")
(#x811966d79dd98f1946929c9137304d5d . "./code.cg2js/cg-edit-in-place.fasl")
(#x944cde059c5f8924623a2f1bf0072d80 . "./code.cg2js/cg-radio-button.fasl")
(#x897643b66bb1b177dd8c9c9e29cbb2f5 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xd7c95a616032f435a5c9d66992e4620c . "./code.cg2js/cg-nodes.fasl")
(#xf1048532fcb1cfb13c6ea303751fb8d8 . "./code.cg2js/cg-get-position.fasl")
(#x0ac8699717e472f5434ae6139a1931dd . "./code.cg2js/cg-directory-dialog.fasl")
(#x58d13286bae5419d3fa407676c013679 . "./code.cg2js/cg-lisp-text.fasl")
(#x9ed5774eabfcc02111e60fccd019dab8 . "./code.cg2js/cg-combo-box.fasl")
(#x40e388e0f09c6912d6ea7575875a4723 . "./code.cg2js/cg-text-widget.fasl")
(#x2071272db672c2ba11080045f4ad78f0 . "./code.cg2js/cg-list-view.fasl")
(#x0865c5d0e1a7dd6c4ca8698837f6abff . "./code.cg2js/cg-graphics-context.fasl")
(#xa857ac1745c9d45c472292b93086d72d . "./code.cg2js/cg-get-pixmap.fasl")
(#x55876accd1072344e3e86b3120f3968c . "./code.cg2js/cg-menu.fasl")
(#xd63cb7067280aba544449673097be6c4 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x71b25c26abee9ae593742adc502ff1d7 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x064a0d46df5f5b462f1d1a3ec50f7f40 . "./code.cg2js/cg-tooltip.fasl")
(#x6836e960b7cd6d62644504503d03d2cc . "./code.cg2js/cg-html-widget.fasl")
(#x391bf1899ac945839f2bcd3059c888b7 . "./code.cg2js/cg-choice-list.fasl")
(#x6faf52d373a808ad911cdd1aaf449721 . "./code.cg2js/cg-icon.fasl")
(#xe3edb8ae1a1fa5c2f920adbcda196cd9 . "./code.cg2js/cg-acache.fasl")
(#xb788f3643325021bc392b96d3d2f367b . "./code.cg2js/cg-template-string.fasl")
(#x6fc19f18322bbf1f2503a5f6d2390988 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x57f59ff587e634140f63bdafd0a5e6f0 . "./code.cg2js/cg-toggling-widget.fasl")
(#x249cf099e2f34bbe97967832c361d9b1 . "./code.cg2js/cg-tab-control.fasl")
(#x737aa7f282acb6ce812171772df8cb30 . "./code.cg2js/cg-split-bar.fasl")
(#x4655296c3497b46e20d67d953fcebe7f . "./code.cg2js/cg-picture-widget.fasl")
(#xd7a9130b396e1ac0bba1441ee94ddfd4 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xf6b7d00ed8edad64f7c63e91f00f6600 . "./code.cg2js/cg-choose-printer.fasl")
(#x1a3f1021a81b5a9481d626ecf175a746 . "./code.cg2js/cg-os-window.fasl")
(#xc13c00f2308b51948400781c5a39e735 . "./code.cg2js/cg-scroll-bar.fasl")
(#x47f084376cc23ddab43471eb0f62c0b3 . "./code.cg2js/cg-clipboard.fasl")
(#x79814a4eaf121720ebc686883214891e . "./code.cg2js/cg-find-string-dialog.fasl")
(#x2bc3c049f5f16fbea3dcb514c9c4954b . "./code.cg2js/cg-hotspot.fasl")
(#x0ad7f87e181d171b19f47cdd2a7af0d6 . "./code.cg2js/cg-check-box.fasl")
(#xb26def47bd788fb1a1090919c040ea4d . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x982d85a2a397d46ac72b6a086e953dfd . "./code.cg2js/cg-file-dialog.fasl")
(#x152a5d3be19394d1e1789b5e3312521f . "./code.cg2js/cg-toolbar.fasl")
(#x31aa96857114e60ea0ab57a6773d247e . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x621db99c11cbf318033b69ba4928a91a . "./code.cg2js/cg-chart-or-plot.fasl")
(#x2680543c7d8535594a0788026d088f11 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xb50a9cefd564361ed1cd38eac02c9f25 . "./code.idejs/ide-project.fasl")
(#x5ddf4d11a994767ac130216b2ac5c646 . "./code.idejs/ide-intro.fasl")
(#x4b75a311a27cc07679baf997899a654b . "./code.idejs/ide-find-in-files.fasl")
(#x5a2f5e102c33505c2bb3d6e782f79cb7 . "./code.idejs/ide-grapher.fasl")
(#x8a6251e3eb282f219405c36c2055d79c . "./code.idejs/ide-inspector.fasl")
(#x3439601cb65d06ac307d9ed015e26e9e . "./code.idejs/ide-editor.fasl")
(#x5d6266a3e9a28880385064c727c38ff0 . "./code.idejs/ide-window-list.fasl")
(#x571f56387d196d1123720e183b6abed8 . "./code.idejs/ide-process-browser.fasl")
(#x9bb433226ddef97f85ece6c8b04e0c14 . "./code.idejs/ide-patch-dialog.fasl")
(#x3e66ef91470a0732111308d1ba97ac7e . "./code.idejs/ide-class-grapher.fasl")
(#xf0f28ce2af6216779e150cd6442fccca . "./code.idejs/ide-base.fasl")
(#xf7988f6e1097e768b6151ecb5d2a3f06 . "./code.idejs/ide.fasl")
(#x58191e16a83970c21301c1212f4af8b3 . "./code.idejs/ide-trace.fasl")
(#x24e4d55ca5c6b74d370f97aa4a433fb7 . "./code.idejs/ide-form.fasl")
(#xa7b38abbdda455be4375c77dd6df7912 . "./code.idejs/ide-find-definitions.fasl")
(#x0506c4bf567719fc0ca4113f472cf30e . "./code.idejs/ide-apropos.fasl")
(#xe46f320f15120fdb7cc8df815fcdb36b . "./code.idejs/ide-class-browser.fasl")
(#x4252dc08f77259aeb4e9c4d0759ad041 . "./code.idejs/ide-navigator.fasl")
(#x89eea60f829aaf8bcc0559fccb73f9bd . "./code.idejs/ide-clipboard.fasl")
(#xdfbb96446f940390a09c050080ba4120 . "./code.idejs/ide-acache.fasl")
(#xdb1d9bcd791f042e8af429549048f855 . "./code.idejs/ide-keyshort.fasl")
(#x576b52b63bdde271370ec779f90df774 . "./code.idejs/ide-package-list.fasl")
(#x369dc0493f42430e01d44ee5357b09c9 . "./code.idejs/ide-dev.fasl")
(#x49a17b9a618997e974ff2e9af0f2eb06 . "./cgjs.html")
(#x44f169492307df5920f609d11cc1a34c . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
